@@ -1,15 +1,16 @@
 package Proc::Queue;
 
-require 5.005_62;
+require 5.005;
+
+our $VERSION = '1.14_01';
+
 use strict;
-use warnings;
+# use warnings;
 require Exporter;
 use Carp;
-
 use POSIX ":sys_wait_h";
 
 our @ISA = qw(Exporter);
-
 our %EXPORT_TAGS = ( all => [ qw( fork_now
 				  waitpids
 				  run_back
@@ -18,17 +19,18 @@ our %EXPORT_TAGS = ( all => [ qw( fork_now
 				  system_back_now
 				  all_exit_ok
 				  running_now ) ] );
-
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
 
-our $VERSION = '0.09';
+# are we running on Windows?
+use constant _win => $^O=~/win32/i;
 
 # parameters
 my $queue_size=4; # max number of councurrent processes running.
 my $debug=0; # shows debug info.
-my $trace=0; # shows function enters.
+my $trace=0; # shows function calls.
 my $delay=0; # delay between fork calls.
+my $ignore_childs=0; # similar to $SIG{CHILD}='IGNORE';
 
 my $last=0; # last time fork was called.
 
@@ -44,12 +46,13 @@ my @captured;
 sub import {
   my ($pkg,@opts)=@_;
   my $i;
-  for ($i=0; $i<=$#opts; $i++) {
+  for ($i=0; $i<@opts; $i++) {
     my $o=$opts[$i];
     if( $o eq 'size'
         or $o eq 'debug'
         or $o eq 'trace'
-	or $o eq 'delay' ) {
+	or $o eq 'delay'
+	or $o eq 'ignore_childs') {
       $#opts>$i or croak "option '$o' needs a value";
       my $value=$opts[$i+1];
       { no strict qw( subs refs );
@@ -63,63 +66,70 @@ sub import {
 }
 
 sub delay {
-  my $d=shift;
+  return $delay unless @_;
   my $old_delay=$delay;
-  if(defined $d) {
-    $delay=$d;
-    carp "Proc queue delay set to $delay sec., it was $old_delay" if $debug;
-  }
+  $delay=$_[0];
+  carp "Proc queue delay set to ${delay}s, it was $old_delay" if $debug;
   return $old_delay;
 }
 
 sub size {
+  return $queue_size unless @_;
   my $size=shift;
   my $old_size=$queue_size;
-  if(defined $size) {
-    croak "invalid value for Proc::Queue size ($size), min value is 1"
-      unless $size >= 1;
-    $queue_size=$size;
-    carp "Proc queue size set to $queue_size, it was $old_size" if $debug;
-  }
+  croak "invalid value for Proc::Queue size ($size), min value is 1"
+    unless $size >= 1;
+  $queue_size=$size;
+  carp "Proc queue size set to $size, it was $old_size" if $debug;
   return $old_size;
 }
 
 sub debug {
-  my $d=shift;
+  return $debug unless @_;
   my $old_debug=$debug;
-  if(defined $d) {
-    if ($d) {
-      $debug=1;
-      carp "Debug mode is now ON for Proc::Queue module";
-    }
-    else {
-      $debug=0;
-      carp "Debug mode is now OFF for Proc::Queue module" if $old_debug;
-    }
+  if ($_[0]) {
+    $debug=1;
+    carp "debug mode ON";
+  }
+  else {
+    $debug=0;
+    carp "debug mode OFF" if $old_debug;
   }
   return $old_debug;
 }
 
 sub trace {
-  my $t=shift;
+  return $trace unless @_;
   my $old_trace=$trace;
-  if(defined $t) {
-    if ($t) {
-      $trace=1;
-      carp "Trace mode is now ON for Proc::Queue module" if $debug;
-    }
-    else {
-      $trace=0;
-      carp "Trace mode is now OFF for Proc::Queue module" if $debug;
-    }
+  if ($_[0]) {
+    $trace=1;
+    carp "trace mode ON" if $debug;
+  }
+  else {
+    $trace=0;
+    carp "trace mode OFF" if $debug;
   }
   return $old_trace;
 }
 
+sub ignore_childs {
+  return $ignore_childs unless @_;
+  my $old_ignore_childs=$ignore_childs;
+  if ($_[0]) {
+    $ignore_childs=1;
+    carp "ignore_childs mode ON" if $debug;
+  }
+  else {
+    $ignore_childs=0;
+    carp "ignore_childs mode OFF" if $debug;
+  }
+  return $old_ignore_childs;
+}
+
 # sub to store internally captured processes
 sub _push_captured {
-  push @captured, shift,$?;
-  croak "captured stack is corrupted" unless ($#captured & 1)
+  push @captured, shift,$? unless $ignore_childs;
+  croak "captured stack is corrupted" if (@captured & 1)
 }
 
 # do the real wait and housekeeping
@@ -148,7 +158,7 @@ sub new_wait () {
   if(@captured) {
     my $w=shift @captured;
     $?=shift @captured;
-    carp "Wait returning old child $w captured in fork" if $debug;
+    carp "Wait returning old child $w captured in wait" if $debug;
     return $w;
   }
   return _wait;
@@ -177,7 +187,11 @@ sub _waitpid ($$) {
 
 sub _clean() {
   my $pid;
-  _push_captured($pid) while(($pid=_waitpid(-1,WNOHANG))>0);
+  while (1) {
+    $pid=_waitpid(-1,WNOHANG);
+    return unless ((_win && $pid < -1) || $pid >0);
+    _push_captured $pid
+  }
 }
 
 sub new_waitpid ($$) {
@@ -196,20 +210,19 @@ sub new_waitpid ($$) {
 }
 
 sub new_exit (;$ ) {
-  my $e=shift;
-  carp "Proc::Queue::exit($e) called" if $trace;
-  carp "Process $$ exiting with value $e" if $debug;
+  my $e=@_?shift:0;
+  carp "Proc::Queue::exit(".(defined($e)?$e:'undef').") called" if $trace;
+  carp "Process $$ exiting with value ".(defined($e)?$e:'undef') if $debug;
   return CORE::exit($e);
 }
 
-
 # use Time::Hires::time if available;
 BEGIN { eval "use Time::HiRes 'time'" }
 
 sub _fork () {
   carp "Proc::Queue::_fork called" if $trace && $debug;
   if ($delay>0) {
-    my $wait=$last+$delay - time();
+    my $wait=$last+$delay - time;
     if ($wait>0) {
       carp "Delaying $wait seconds before forking" if $debug;
       select (undef,undef,undef,$wait);
@@ -241,7 +254,7 @@ sub _fork () {
 sub new_fork () {
   carp "Proc::Queue::fork called" if $trace;
   while($queue_now>=$queue_size) {
-    carp "Waiting that some process finishes before continuing" if $debug;
+    carp "Waiting for some process to finish" if $debug;
     my $nw;
     if (($nw=_wait) != -1) {
       _push_captured $nw;
@@ -251,7 +264,7 @@ sub new_fork () {
       last;
     }
   }
-  return _fork();
+  return _fork;
 }
 
 sub fork_now () {
@@ -266,7 +279,7 @@ sub waitpids {
     if (defined($pid)) {
       carp "Waiting for child $pid to exit" if $debug;
       my $r=new_waitpid($pid,0);
-      if ($r > 0) {
+      if ((_win && $r < -1) || $r > 0) {
 	carp "Child $r return $?" if $debug;
 	push @result,$r,$?;
       }
@@ -344,10 +357,10 @@ sub all_exit_ok {
   carp "Proc::Queue::all_exit_ok(".join(", ",@_).")" if $trace;
   my @result=waitpids(@_);
   my $i;
-  for($i=0;$i<=$#result;$i++) {
+  for($i=0;$i<@result;$i++) {
     next unless $i&1;
     if (!defined($result[$i]) or $result[$i]) {
-      carp "Child $_[$i>>1] fail with code $result[$i], waitpid return $result[$i-1]" if $debug;
+      carp "Child ".$_[$i>>1]." fail with code $result[$i], waitpid return $result[$i-1]" if $debug;
       return 0;
     }
   }
@@ -368,25 +381,25 @@ sub running_now () {
 
 
 1;
+
 __END__
 
 # docs:
 
 =head1 NAME
 
-Proc::Queue - Perl extension to limit the number of concurrent child
-process running
+Proc::Queue - limit the number of child processes running
 
 =head1 SYNOPSIS
 
   use Proc::Queue size => 4, debug => 1;
 
   package other;
+  use POSIX ":sys_wait_h"; # imports WNOHANG
 
-  # this loop will create new childs, but Proc::Queue will make it
-  # wait when the limit (4) is reached until some of the old childs
-  # exit.
-  foreach (1..10) {
+  # this loop creates new childs, but Proc::Queue makes it wait every
+  # time the limit (4) is reached until enough childs exit
+  foreach (1..100) {
     my $f=fork;
     if(defined ($f) and $f==0) {
       print "-- I'm a forked process $$\n";
@@ -394,6 +407,7 @@ process running
       print "-- I'm tired, going away $$\n";
       exit(0)
     }
+    1 while waitpid(-1, WNOHANG)>0; # reaps childs
   }
 
   Proc::Queue::size(10); # changing limit to 10 concurrent processes
@@ -402,7 +416,7 @@ process running
   Proc::Queue::delay(0.2); # set 200 miliseconds as minimum
                            # delay between fork calls
 
-  package other; # just to test it works in any package
+  package other; # just to test it works on any package
 
   print "going again!\n";
 
@@ -421,30 +435,53 @@ process running
 
 =head1 DESCRIPTION
 
-This module lets you parallelice a perl program using the C<fork>,
-C<exit>, C<wait> and C<waitpid> calls as usual and without the need to
-take care of creating too much processes and overloading the machine.
+This module lets you parallelise a perl program using the C<fork>,
+C<exit>, C<wait> and C<waitpid> calls as usual but without taking care
+of creating too many processes and overloading the machine.
 
-It works redefining C<fork>, C<exit>, C<wait> and C<waitpid> functions
-so old programs do not have to be modified to use this module (only
-the C<use Proc::Queue> sentence is needed).
+It redefines perl C<fork>, C<exit>, C<wait> and C<waitpid> core
+functions. Old programs do not need to be reprogrammed, only the C<use
+Proc::Queue ...> sentence has to be added to them.
 
-Additionally, the module have two debugging modes (debug and trace)
-that can be activated and that seem too be very useful when developing
-parallel aplications.
+Additionally, the module has two debugging modes (debug and trace)
+that seem too be very useful when developing parallel aplications:
 
-Debug mode when activated dumps lots of information about processes
-being created, exiting, being caught be parent, etc.
+=over 4
 
-Trace mode just prints a line every time one of the C<fork>, C<exit>,
-C<wait> or C<waitpid> functions is called.
+=item debug mode:
 
-It is also possible to set a minimun delay time between calls to fork
-to stop consecutive processes for starting in a short time interval.
+when active, dumps lots of information about processes being created,
+exiting, being caught by parent, etc.
+
+=item trace mode:
+
+prints a line every time one of the C<fork>, C<exit>, C<wait> or
+C<waitpid> functions are called.
+
+=back
+
+It is also possible to set a minimun delay time between fork calls
+to stop too many processes for starting in a short time interval.
+
+Child processes continue to use the modified functions, but their
+queues are reset and the maximun process number for them is set to 1
+(anyway, childs can change their queue size themselves).
+
+Proc::Queue doesn't work if CHLD signal handler is set to
+C<IGNORE>.  
+
+Internally, Proc::Queue, automatically catches zombies and stores
+their exit status in a private hash. To avoid leaking memory in long
+running programs you have to call C<wait> or C<waitpid> to delete
+entries from that hash or alternatively active the C<ignore_childs>
+mode:
+
+  Proc::Queue::ignore_childs(1)
+
+or
+
+  use Proc::Queue ignore_childs=>1, ...
 
-Childs processes continue to use the modified functions, but its
-queues are reset and the maximun process number for them is set to
-1. Althought childs can change it to any other value if needed.
 
 =head2 EXPORT
 
@@ -474,17 +511,41 @@ If no argument is given, the number of processes allowed is returned.
 
 =item delay(), delay($time)
 
-delay lets you set a minimun time in seconds to elapse between every
-consecutive calls to fork. This is usefull for not creating to much
-processes in a very short time.
+lets you set a minimun time in seconds to elapse between consecutive
+calls to fork. It is useful to avoid creating too many processes in
+a short time (that could degrade performance).
 
-If the Time::HiRes module is available it will be used and delays
-shorted that 1 second could be used.
+If Time::HiRes module is available delays shorted that 1 second are
+allowed.
 
 If no arg is given, the current delay is returned.
 
 To clear it use C<Proc::Queue::delay(0)>.
 
+=item ignore_childs($on)
+
+calling
+
+  Proc::Queue::ignore_childs(1);
+
+is the equivalent to
+
+  $SIG{CHLD}='IGNORE'
+
+when using Proc::Queue.
+
+=item debug(), debug($boolean), trace(), trace($boolean)
+
+Change or return the status for the debug and trace modes.
+
+=back
+
+
+Other utility subroutines that can be imported from Proc::Queue are:
+
+
+=over 4
+
 =item fork_now()
 
 Sometimes you would need to fork a new child without waiting for other
@@ -500,8 +561,8 @@ exportable so you can do...
 
 =item waitpids(@pid)
 
-Will wait for all the processes in @pid to exit. It returns an array
-with pairs pid and exit values (pid1, exit1, pid2, exit2, pid3,
+waits for all the processes in @pid to exit. It returns an array
+with pairs of pid and exit values (pid1, exit1, pid2, exit2, pid3,
 exit3,...) as returned by individual waitpid calls.
 
 =item run_back(\&code), run_back { code }
@@ -516,8 +577,8 @@ A mix between run_back and fork_now.
 =item system_back(@command)
 
 Similar to the C<system> call but runs the command in the background
-and waits for some childs to exit first if there are already too many
-childs running.
+and waits for other childs to exit first if there are already too many
+running.
 
 =item system_back_now(@command)
 
@@ -526,22 +587,18 @@ allowed has been reached.
 
 =item all_exit_ok(@pid)
 
-Do a waitpids call and test that all the processes exit with code 0.
+Do a C<waitpids> call and test that all the processes exit with code 0.
 
 =item running_now()
 
 Returns the number of child processes currently running.
 
-=item debug(), debug($boolean), trace(), trace($boolean)
-
-Change or return the status for the debug and trace modes.
-
 =item import(pkg,opt,val,opt,val,...,fnt_name,fnt_name,...)
 
 The import function is not usually explicitally called but by the
-C<use Proc::Queue> statement. The options allowed are C<size>, C<debug>
-and C<trace> and they let you configure the module instead of using
-the C<size>, C<debug> or C<trace> module functions as in...
+C<use Proc::Queue> statement.
+
+Options allowed are C<size>, C<debug> and C<trace>, i.e:
 
   use Proc::Queue size=>10, debug=>1;
 
@@ -550,28 +607,13 @@ function name to be imported.
 
   use Proc::Queue size=>10, ':all';
 
+
 =head2 BUGS
 
-None that I know, but this is just version 0.09!
+Proc::Queue is a very stable module, no bugs have been reported
+for a long time.
 
-The module has only been tested under Solaris 2.6
-
-Child (forking) behaviour althought deterministic could be changed to
-something better. I would accept any suggestions on it.
-
-=head1 INSTALL
-
-As usual, unpack de module distribution and from the newly created
-directory run:
-
-  $ perl Makefile.PL
-  $ make
-  $ make test
-  $ make install
-
-=head1 AUTHOR
-
-Salvador Fandino <sfandino@yahoo.com>
+Support for Win32 OSs is still experimental.
 
 =head1 SEE ALSO
 
@@ -579,4 +621,17 @@ L<perlfunc(1)>, L<perlipc(1)>, L<POSIX>, L<perlfork(1)>,
 L<Time::HiRes>, L<Parallel::ForkManager>. The C<example.pl> script
 contained in the module distribution.
 
+
+=head1 AUTHOR
+
+Salvador Fandio E<lt>sfandino@yahoo.comE<gt>
+
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright 2001, 2002, 2003 by Salvador Fandio
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
+
 =cut
